import { NextRequest, NextResponse } from 'next/server';
import { API_BASE_URL } from '@/utils/api-config';

/**
 * Health Check Endpoint für Frontend-Server
 * GET /health
 */
export async function GET(request: NextRequest) {
  try {
    // Bestimme die Backend-URL basierend auf der Konfiguration
    const backendUrl = typeof API_BASE_URL === 'object' ? API_BASE_URL.primary : API_BASE_URL;
    let backendStatus = 'unknown';
    
    try {
      // AbortController für Timeout verwenden
      const controller = new AbortController();
      const timeoutId = setTimeout(() => controller.abort(), 5000);
      
      const backendResponse = await fetch(`${backendUrl}/health`, {
        method: 'GET',
        signal: controller.signal,
      });
      
      clearTimeout(timeoutId);
      backendStatus = backendResponse.ok ? 'connected' : 'error';
    } catch {
      backendStatus = 'disconnected';
    }

    return NextResponse.json({
      status: 'healthy',
      service: 'myp-frontend',
      timestamp: new Date().toISOString(),
      version: '1.0.0',
      backend: {
        url: backendUrl,
        status: backendStatus
      },
      environment: process.env.NODE_ENV || 'development'
    }, { status: 200 });

  } catch (error) {
    return NextResponse.json({
      status: 'unhealthy',
      service: 'myp-frontend',
      timestamp: new Date().toISOString(),
      error: error instanceof Error ? error.message : 'Unknown error'
    }, { status: 503 });
  }
} 